# sharcdiag.tcl---SHARC Diagnostics frame

set debug 0
set started 0

set frame SHARCDIAG
set frametitle "SHARC & Collator Diagnostics"
set icontitle "SHARCDiag"

#    current server resource
set resource "mem"
#    list of available server resources
set resources {}
#    number of available SHARC server resources
set numresources 0

# These values must match /eurogam/great/include/great.h
set ITEM_BYTES 8
set CHUNK_LEN 8
set CHUNK_BYTES [expr $CHUNK_LEN * $ITEM_BYTES]
set COLLATE_BUFFER_LEN [expr 320 * 1024]
set COLLATE_BUFFER_CHUNKS [expr $COLLATE_BUFFER_LEN / $CHUNK_LEN]
set DMA_BUFFER_BYTES 0x00100000
set SHARC_BUFFER_BYTES [expr 128 * $ITEM_BYTES]

set ASPlistsharc0 [list 0 1 2 3]
set ASPlistsharc1 [list 4 5 6 7]
set CurrentASP 0

proc make-resource-list {} {
  global env resource resources numresources

   set z [catch {eg "(experiment ?resources)"} m]
   if {$z==0} then {set rs $m} else {set rs ""}

#    scan through resources and count instances of all mem register servers
   if {[info exists env(SHARCServer)] == 0} then {set role mem}  \
      else {set role $env(SHARCServer)}
   set count 0
   set rolecount(mem) 0
   foreach r $rs {
      set type [second [fourth $r]]
      set role [second [fifth $r]]
if {[second [first $r]] != "merge"} then {
      if {$type == "register" && $role == "mem"} then {incr count; incr rolecount(mem)}
}
   }

   if {[info exists env(SHARCServer)] == 0} then {set resource mem}  \
      else {set resource $env(SHARCServer)}
   set resources {}
   set numresources 0
   if {$rolecount(mem) > 0}   then {lappend resources mem; incr numresources}

   if {$count > 1 || $role != $resource} then {
#    next scan through resources and add instances of all mem register servers to the list
      foreach r $rs {
         set type [second [fourth $r]]
         set role [second [fifth $r]]
         set name [second [first $r]]
if {$name != "merge"} then {
         if {$type == "register" && $role == "mem" && $rolecount(mem) > 1}   then {lappend resources $name; incr numresources}
}
      }
   } else {
      if {[info exists env(SHARCServer)] == 0} then {set resources mem}  \
         else {set resources $env(SHARCServer)}
      set numresources 1
   }
   if {[info exists env(SHARCServer)] == 0} then {set resource [first $resources]}  \
      else {set resource $env(SHARCServer)}

   make-processor-list
}


proc click arg {
   global debug hsdir
   if {$debug} then {insert-debug "click with argument \{$arg\}"}
   set f [lindex $arg 0]
   set c [lindex $arg 1]
   set w [lindex $arg 2]
   set v [lindex $arg 3]
   if {$c==":quit"} then {click-quit $f; return}
   switch -glob $w {
      DET-*             {click-overlay $w $v}
      default    {puts stdout $arg}
   }
}

proc click-quit {f} {
   global env started
   if {$f=="SEL"} then {destroy-frame SEL; return}
   destroy-frame SEL
   close-frame
}

proc finish args {destroy-frame SEL; destroy-frame; set started 0}

set buffnames {"Merge Output" "Merge Input" "Collate Output" "Link Queue" "Collate Input" "SHARC Link"}

set timing 0

set sname SHARC
set cname Collate
set mname Merge
set MergeProc merge

set ProcIndex 0
set CurrentProc mem
set ProcessorList {mem}
set HostList {mem}

set sharcmem 0

set SourceIndex 0
set LinkIndex 0

set meminc 0

set dumpstart 0
set dumpwords 64
set dumpdata  ""

set memOffset 0x0000

#set PMC_SCTL		0xc2000000
#set SHARC_IOP		0xc2200000
set SHARC_PIO		0x00

set SHARC_IOP_OFFSET	0x00200000

set SHARC_SHARC_BASE	0x28400
#set LINK_OVERRUN_TOP	0x12
set PCTL_MBOX_BASE	0x40

# base address of link buffer 0
set SHARC_LK_BUFF_BASE	0x20400
set SHARC_LK_BUFF_BASE	0x27c00

set IOP_SYSCON		0x00000000
set ASP_SYSCON		0x0000e000
# for 48-bit access
set ASP_SYSCON_LK_RD	0x0000e028
	
for {set i 0} {$i<8} {incr i} {
   set PCTL_MBOX$i [format "0x%x" [expr $PCTL_MBOX_BASE + (4*$i)]]
}

set PCTL_PCI2SHARC 0x60
set PCTL_SHARC2PCI 0x64

set PPC_SHARC_BASE [format "0x%08x" [expr $SHARC_IOP_OFFSET + ($SHARC_SHARC_BASE << 2)]]
set PPC_LK_BUFF_BASE [format "0x%08x" [expr $SHARC_IOP_OFFSET + ($SHARC_LK_BUFF_BASE << 2)]]

for {set i 0} {$i<4} {incr i} {
#   set LINK_OVERRUN$i [format "0x%02x" [expr ($LINK_OVERRUN_TOP + $i) << 2]]
   set LINK_BUFFER$i [format "0x%02x" [expr ($i * 256) << 2]]
}

#set DMA_BUF [format "0x%08x" [expr 64*1024*1024 - 64*1024]]
set DMA_BUF 0

set barRegisters [list \
   "_SCTL generic -n -t int -fx" \
   "_PCTL generic -n -t int -fx" \
   "_DMA generic -n -t int -fx" \
]

set DMARegister   "DMABuffer bMEM dma -a 0 -o$memOffset -r0" 

#   "Link.OverRun0 xMEM sctl -a $PPC_SHARC_BASE -o$LINK_OVERRUN0 -zd -r1" \
#   "Link.OverRun1 xMEM sctl -a $PPC_SHARC_BASE -o$LINK_OVERRUN1 -zd -r1" \
#   "Link.OverRun2 xMEM sctl -a $PPC_SHARC_BASE -o$LINK_OVERRUN2 -zd -r1" \
#   "Link.OverRun3 xMEM sctl -a $PPC_SHARC_BASE -o$LINK_OVERRUN3 -zd -r1" \

set ASPC2Registers [list \
   "DMABuffer bMEM dma -a $DMA_BUF -o$memOffset -r0"  \
   "LinkBuffer0 bMEM sctl -a $PPC_LK_BUFF_BASE -o$LINK_BUFFER0 -zd -r1"  \
   "MailBox0 xMEM pctl -a $SHARC_PIO -o$PCTL_MBOX0 -r1 -ro" \
   "MailBox1 xMEM pctl -a $SHARC_PIO -o$PCTL_MBOX1 -r1 -ro" \
   "MailBox2 xMEM pctl -a $SHARC_PIO -o$PCTL_MBOX2 -r1 -ro" \
   "MailBox3 xMEM pctl -a $SHARC_PIO -o$PCTL_MBOX3 -r1 -ro" \
   "MailBox4 xMEM pctl -a $SHARC_PIO -o$PCTL_MBOX4 -r1 -ro" \
   "MailBox5 xMEM pctl -a $SHARC_PIO -o$PCTL_MBOX5 -r1 -ro" \
   "MailBox6 xMEM pctl -a $SHARC_PIO -o$PCTL_MBOX6 -r1 -ro" \
   "MailBox7 xMEM pctl -a $SHARC_PIO -o$PCTL_MBOX7 -r1 -ro" \
   "PCI2Sharc xMEM pctl -a $SHARC_PIO -o$PCTL_PCI2SHARC -r1" \
   "Sharc2PCI xMEM pctl -a $SHARC_PIO -o$PCTL_SHARC2PCI -r1" \
   "Sharc.IOP_SysCon xMEM sctl -a $SHARC_IOP_OFFSET -o$IOP_SYSCON -m16 -zx -r1" \
   "Sharc.Memory xMEM sctl -a $PPC_SHARC_BASE -o0 -zx -r1" \
   "Link.Buffer0 xMEM sctl -a $PPC_LK_BUFF_BASE -o$LINK_BUFFER0 -zd -r1" \
   "Link.Buffer1 xMEM sctl -a $PPC_LK_BUFF_BASE -o$LINK_BUFFER1 -zd -r1" \
   "Link.Buffer2 xMEM sctl -a $PPC_LK_BUFF_BASE -o$LINK_BUFFER2 -zd -r1" \
   "Link.Buffer3 xMEM sctl -a $PPC_LK_BUFF_BASE -o$LINK_BUFFER3 -zd -r1"
]

set table(DET-P2S)      {write-sharc PCI2Sharc}
set table(DET-S2P)      {write-sharc Sharc2PCI}

proc click-on {w v} {
	global debug table  

	if {$debug} then {puts stderr "click on $w $v"}
	set func [first $table($w)]
	set reg  [second $table($w)]

	$func $reg $v
}

proc click-overlay {w v} {
   global memOffset meminc SourceIndex
#      DET-PROCESSOR	{click-processor $v}
#      DET-DMA2LINKQ	{click-stophere $w $v}
#      DET-LINKQ2OLDEST	{click-stophere $w $v}
#      DET-OLDEST2COLLATE	{click-stophere $w $v}
#      DET-COLLATE2MERGE	{click-stophere $w $v}
#      DET-PASSTHRU	{click-stophere $w $v}

   switch -glob $w {
      DET-ASP		{click-asp $v}
      DET-MEMADDR	{click-memaddr $v;update-sharcmem}
      DET-SOURCE	{click-source $v}
      DET-LINK		{click-link $v}
      DET-MEMOFFSET	{click-offset $v}
      DET-INCR		{click-incr $v}
      DET-EMPTY		{:modify DET-MEMDISPLAY :empty T;global dumpdata; set dumpdata  ""}
      DET-REDISPLAY	{click-redisplay}
      DET-P2S		-
      DET-S2P		{click-on $w $v}
      default	{puts stdout "$w $v"}
   }
}

proc click-redisplay {} {
   global  frame
   if {![winfo ismapped [:path $frame]]} then {return}
   make-processor-list
   update-sharc
#   update-sharcmem
   update-others
#   update-stophere
   update-datacounters
}

proc update-sharcmem {} {
   global sname sharcmem PPC_SHARC_BASE
   erswta $sname.Sharc.Memory -o [expr $PPC_SHARC_BASE + ($sharcmem << 2)]
   set v [ersread $sname.Sharc.Memory]
   :modify DET-MEMCONT :value $v
}

proc click-processor {n} {
  global ProcIndex CurrentProc 
  set ProcIndex $n


  make-processor-list
  global ASPlist$CurrentProc
  :modify DET-ASP :strings [set ASPlist$CurrentProc]

  click-asp 0
  click-redisplay
}

proc click-asp {n} {
  global ProcIndex CurrentProc CurrentASP

  if {$n > 3} then {
    set n [expr $n - 4]
    set ProcIndex 1
   } {set ProcIndex 0}

  make-processor-list
  global ASPlist$CurrentProc
  
  set CurrentASP [lindex [set ASPlist$CurrentProc] $n]
  do-define-asp-registers $CurrentASP
  click-redisplay
}

proc click-memaddr {v} {
   global sharcmem
   set sharcmem $v
}

proc do-memmod {v} {
   global sharcmem
   incr sharcmem $v
   :modify DET-MEMADDR :value [format 0x%08x $sharcmem]
}

proc click-source {n} {
   global SourceIndex LinkIndex dumpwords
   set OldIndex $SourceIndex
   set SourceIndex $n
   if {$SourceIndex==5} then {
      set label "Read Link"
      set dumpwords 128
   } else {
      set label "Read Buffer"
      set dumpwords 64
      if {$OldIndex != $SourceIndex} then {zero_offset}
   }
   if {$SourceIndex==3} then {
      :modify DET-RDBUFF :inactive T
   } else {
      :modify DET-RDBUFF :inactive F
   }
   if {$SourceIndex==3} then {
      :modify DET-RDCURR :inactive T
   } else {
      :modify DET-RDCURR :inactive F
   }
   :modify DET-RDBUFF :label $label
   update-others
}

proc click-link {n} {
  global LinkIndex

  set LinkIndex $n
}

proc click-offset {v} {
   set_offset $v
}

proc click-incr {v} {
   global SourceIndex meminc
   set m 0
   if {$SourceIndex!=5} then {
      set meminc $v
      set m $v
   }
   :modify DET-INCR :value $m
}

proc xclick-stophere {w v} {
   global cname CurrentASP
   :modify DET-DMA2LINKQ     :value 0
   :modify DET-LINKQ2OLDEST :value 0
   :modify DET-OLDEST2COLLATE :value 0
   :modify DET-COLLATE2MERGE :value 0
   :modify DET-PASSTHRU :value 0
   :modify $w :value $v
   if {$v} then {
     switch -glob $w {
       DET-DMA2LINKQ	 {
            :modify DET-LINKQ2OLDEST :value 0
            :modify DET-OLDEST2COLLATE :value 0
            :modify DET-COLLATE2MERGE :value 0
            :modify DET-PASSTHRU :value 0
            set v 1
       }

       DET-LINKQ2OLDEST	 {
            :modify DET-DMA2LINKQ     :value 0
            :modify DET-OLDEST2COLLATE :value 0
            :modify DET-COLLATE2MERGE :value 0
            :modify DET-PASSTHRU :value 0
            set v 2
       }

       DET-OLDEST2COLLATE	 {
            :modify DET-DMA2LINKQ     :value 0
            :modify DET-LINKQ2OLDEST :value 0
            :modify DET-COLLATE2MERGE :value 0
            :modify DET-PASSTHRU :value 0
            set v 3
       }

       DET-COLLATE2MERGE	 {
            :modify DET-DMA2LINKQ     :value 0
            :modify DET-LINKQ2OLDEST :value 0
            :modify DET-OLDEST2COLLATE :value 0
            :modify DET-PASSTHRU :value 0
            set v 4
       }

       DET-PASSTHRU	 {
            :modify DET-DMA2LINKQ     :value 0
            :modify DET-LINKQ2OLDEST :value 0
            :modify DET-OLDEST2COLLATE :value 0
            :modify DET-COLLATE2MERGE :value 0
            set v 5
       }
     }
   }
   erswrite $cname.Asp$CurrentASP.StopHere $v
}

proc set_offset {v} {
   global memOffset SourceIndex
   set m 0x0000
   if {$SourceIndex!=5} then {
      set memOffset [format 0x%04x $v]
      set m $memOffset
   }
   :modify DET-MEMOFFSET :value $m
}

proc zero_offset {} {set_offset 0}

proc format_the_dump {} {
  global dumpdata dumpstart dumpwords

#    format TDR data held in global variable dumpdata

   if {$dumpdata == ""} {
      return
   }

   set-footer "formatting data dump"

   set frame "dump"
   set win [self frame-name].$frame
   :frame $win :label  "Formatted Data Dump"  :transient T
   :layout T

   :galley G :rows 50 :cols 75 :strings "" :font fixed
   :show T
   :modify G :strings "dump starting at word offset $dumpstart for $dumpwords words\n\n"

   set waslink 0
   set lasttype 0
   set lastcode 0
   for {set i 0} {$i < $dumpwords} {incr i} {
      set word  [lindex $dumpdata $i]
      set type  [expr $word >> 30 & 3]

      switch $type {
         0 {
            if {$lasttype == "0"} then {
               :modify G :strings "ALIGNMENT ERROR \n"
            }

            set title2 ""
            if {[expr $word  & 0xf0000000] == 0} then {
               if {$waslink != "1"} {
                 set title "Timestamp"
               } else {
                 set title "Sequence Number"
                 set title2 [expr $word]
               }
               if {$lasttype == "2"} {
                 if {$lastcode == "4" || $lastcode == "3" || $lastcode == "2"} {
                  set title2 [format "%05x%07x" $info $word]
                 }
               }
            } else {
               set title "Unrecognised"
            }
            :modify G :strings "[format "%08x %29s %13s" $word $title $title2] \n"
            set waslink 0
           }

         2 {
            if {$lasttype != "0"} then {
               :modify G :strings "ALIGNMENT ERROR \n"
            }

            set mod   [expr $word >> 24 & 0x3f]
            set code  [expr $word >> 20 & 0xf]
            set info  [expr $word & 0xfffff]
            set title3 ""
            switch $code {
            1   {
                 set title "Pileup"
                 set title2 [format "Module %3d" $mod]
                 set title3 [format "Channel %2d" $info]
                 set waslink 0
                }

            2   {
                 set title "Pause"
                 set title2 [format "Module %3d" $mod]
                 set waslink 0
                }

            3   {
                 set title "Resume"
                 set title2 [format "Module %3d" $mod]
                 set waslink 0
                }

            4   {
                 set title "SYNC100"
                 set title2 [format "Module %3d" $mod]
                 set waslink 0
                }

            15  {
                 set title "Link"
                 if {$mod != "63"} {
                   set title2 "Faulty"
                   set waslink 0
                 } else {
                   set title2 [format "Number %d" $info]
                   set waslink 1
                 }
                }

            default  {
                 set title "Unknown Info"
                 set title2 ""
                 set waslink 0
                }
              }
            set lastcode $code
            :modify G :strings "[format "%08x %d %2d %2d  %05x %14s %11s %11s" $word $type $mod $code $info $title $title2 $title3] \n"
           }

         3 {
            if {$lasttype != "0"} then {
               :modify G :strings "ALIGNMENT ERROR \n"
            }

            set title "ADC Data"
            set fail  [expr $word >> 29 & 1]
            if {$fail} {set f "Fail"} {set f "    "}
            set veto  [expr $word >> 28 & 1]
            if {$veto} {set v "Veto"} {set v "    "}
            set id    [expr $word >> 16 & 0xfff]
            set data  [expr $word & 0xffff]

            set title2 [format "Module %3d" [expr $word >> 21 & 0x7f]]
            set title3 [format "Channel %2d" [expr $word >> 16 & 0x1f]]

            :modify G :strings "[format "%08x %d %d %d %03x %04x %14s %11s %11s %5s %s" $word $type $fail $veto $id $data $title $title2 $title3 $f $v] \n"
            set waslink 0
           }

          default  {
            set title "Unrecognised"
            set title2 ""
            :modify G :strings "[format "%08x %29s" $word $title] \n"
            set waslink 0
           }
      }
      set lasttype $type
   }
   clear-footer
}

proc displayDump {header addr inc} {
   global dumpdata dumpwords

   :modify DET-MEMDISPLAY :strings "\n     $header"

   for {set i 0} {$i<$dumpwords} {incr i} {
      if {![expr $i % 8]} {
         :modify DET-MEMDISPLAY :strings "\n"
         :modify DET-MEMDISPLAY :strings [format "0x%06x: " $addr]; incr addr $inc
      }
      :modify DET-MEMDISPLAY :strings [format "%08x " [lindex $dumpdata $i]]
   }
   :modify DET-MEMDISPLAY :strings "\n"
}

proc readMemory {r} {
   global memOffset meminc
   global dumpdata dumpwords

   erswta $r "-o $memOffset -l $dumpwords"
   set dumpdata [string trimleft [ersread $r]]

   displayDump "Memory Offset $memOffset" $memOffset 32

   if {$meminc != 0} then {
      global SourceIndex
      set offset [expr $memOffset + 256]
# The output buffer fills and starts again at 255 items (8 bytes each)
      if {$SourceIndex==0 && $offset>[expr 255 * 8]} then {set offset 0}
      set_offset $offset
   }
}

proc readLink {l} {
   global sname ASP_SYSCON_LK_RD ASP_SYSCON LINK_BUFFER$l
   global dumpdata dumpwords

#   set LINK_BUFFER [expr $PPC_LK_BUFF_BASE + [set LINK_BUFFER$l]]
   set LINK_BUFFER [set LINK_BUFFER$l]

   erswta $sname.Sharc.IOP_SysCon -m16
   erswrite $sname.Sharc.IOP_SysCon $ASP_SYSCON
   erswta $sname.Link.Buffer$l -o $LINK_BUFFER
   ersread $sname.Link.Buffer$l
   ersread $sname.Link.Buffer$l

   set dumpdata ""
   for {set i 0} {$i<$dumpwords} {incr i} {
      append dumpdata "0x[format %08x [expr [ersread $sname.Link.Buffer$l]]] "
      erswta $sname.Link.Buffer$l -o [incr LINK_BUFFER 4]
   }
   erswrite $sname.Sharc.IOP_SysCon $ASP_SYSCON
   erswta $sname.Sharc.IOP_SysCon -m0
   erswrite $sname.Sharc.IOP_SysCon $ASP_SYSCON

   displayDump "Link Buffer $l" 0 8
}

proc click-readBuff {} {
   global SourceIndex LinkIndex
   if {$SourceIndex==5} then {
      readLink $LinkIndex
   } else {
      global CurrentProc MergeProc sname cname mname CurrentASP
      set TempProc $CurrentProc
      switch $SourceIndex {
         0 {set r $mname.Output.DataBuffer; set CurrentProc $MergeProc}
         1 {set r $mname.Asp$CurrentASP.DataBuffer; set CurrentProc $MergeProc}
         2 {set r $cname.Asp$CurrentASP.DataBuffer}
         3 {set r $cname.Asp$CurrentASP.Link$LinkIndex.QData}
         4 {set r $sname.DMABuffer}
         default {set r ""}
      }
      if {"$r" != ""} {readMemory $r}
      set CurrentProc $TempProc
   }

   update-sharc
   update-datacounters
}

proc click-readCurr {} {
   global SourceIndex LinkIndex
   if {$SourceIndex==5} then {
      readLink $LinkIndex
   } else {
      global CurrentProc MergeProc sname cname mname CurrentASP
      set TempProc $CurrentProc
      switch $SourceIndex {
         0 {
            set r $mname.Output.DataBuffer
            set CurrentProc $MergeProc
            set curr [ersread $mname.Output.CurrPointer]
            set_offset $curr
           }

         1 {
            global COLLATE_BUFFER_CHUNKS ITEM_BYTES CHUNK_BYTES
            set r $mname.Asp$CurrentASP.DataBuffer
            set CurrentProc $MergeProc
            set sent [ersread $mname.Asp$CurrentASP.Produced]
            if {$sent != 0} {incr sent -1}
            set_offset [expr ($sent % $COLLATE_BUFFER_CHUNKS) * $CHUNK_BYTES]
           }

         2 {
            global COLLATE_BUFFER_CHUNKS ITEM_BYTES CHUNK_BYTES
            set r $cname.Asp$CurrentASP.DataBuffer
            set CurrentProc $MergeProc
            set sent [ersread $mname.Asp$CurrentASP.Produced]
            if {$sent != 0} {incr sent -1}
            set_offset [expr ($sent % $COLLATE_BUFFER_CHUNKS) * $CHUNK_BYTES]
            set CurrentProc $TempProc
           }

         3 {
            global LinkIndex
            set r $cname.Asp$CurrentASP.Link$LinkIndex.QData
            set q [ersread $cname.Asp$CurrentASP.Link$LinkIndex.QDesc]
            set off [expr [fourth $q] - [first $q]]
            set_offset [expr $off - 256]
           }

         4 {
            global DMA_BUFFER_BYTES SHARC_BUFFER_BYTES
            set r $sname.DMABuffer
            set sent [ersread $sname.MailBox1]
            if {$sent != 0} {incr sent -1}
            set sent [expr $sent * $SHARC_BUFFER_BYTES]
            set_offset [expr $sent % $DMA_BUFFER_BYTES]
           }

         default {set r ""}
      }
      if {"$r" != ""} {readMemory $r}
      set CurrentProc $TempProc
   }

   update-sharc
   update-datacounters
}

proc click-define-asp-registers {} {
   global CurrentASP
   do-define-asp-registers $CurrentASP
}

proc write-sharc {r v} {
   global debug sname

   if {$debug} then {puts stderr "write-sharc $sname.$r $v"}
   erswrite $sname.$r $v
}

proc start {} {
   global started frame CurrentASP
   if {$started==0} then {make-frame} else {open-frame $frame}
   make-resource-list
   catch {do-define-asp-registers $CurrentASP}
   click-redisplay   
}

proc make-frame {} {
   global env frame started
   global ProcessorList buffnames SourceIndex memOffset
   if {$SourceIndex==5} then {
      set label "Read Link"
   } else {
      set label "Read Buffer"
   }

   :frame [self frame-name] :label "[self frame-title]" :show-footer T  :resize F \
        :icon-image "@$env(TCLHOME)/bitmaps/spectrum" \
        :icon-name "[self icon-title]"

# N.B.
# MailBox 0 PCI DMA buffer number
# MailBox 1 Number of buffers sent
# MailBox 2 Address of LUT
# MailBox 3 Date & Version
# MailBox 4 PCI DMA status
# MailBox 5 Unallocated
# MailBox 6 Unallocated
# MailBox 7 Unallocated
#      :text DET-MBOX0  :label "PCI DMA buffer number:" :deaf T :valuex 170 :w 10
#      :text DET-MBOX4  :label "PCI DMA status:" :deaf T :valuex 115 :w 10
#      :next-row T
#      :text DET-MBOX1  :label "Number of buffers sent:" :deaf T :valuex 170 :w 10
#      :text DET-MBOX5  :label "MailBox 5:" :deaf T :valuex 115 :w 10
#      :next-row T
#      :text DET-MBOX2  :label "Address of LUT:" :deaf T :valuex 170 :w 10
#      :text DET-MBOX6  :label "MailBox 6:" :deaf T :valuex 115 :w 10
#      :next-row T
#      :text DET-MBOX3  :label "Date & Version:" :deaf T :valuex 170 :w 10
#      :text DET-MBOX7  :label "MailBox 7:" :deaf T :valuex 115 :w 10


   :panel P20  :layout T :padx 10 :pady 10
      :layout H; :row-gap 10; :col-gap 10
      :text DET-PROCESSOR :label "Register Server" :w 8 :deaf T
      :hskip 32
      :text DET-HOST :label "Host Name" :w 8 :deaf T
      :hskip 32
      :choice DET-ASP :label "ASP" :strings [list 0 1 2 3 4 5 6 7] :exclusive T
      :next-row T

   :panel MBOXES  :layout T :padx 10 :pady 10
      :layout H; :row-gap 0; :col-gap 10
      :text DET-MBOX0  :label "MailBox 0:" :deaf T :valuex 68 :w 10
      :text DET-MBOX1  :label "Buffers sent:" :deaf T :valuex 98 :w 10
      :text DET-MBOX2  :label "Buffers read:" :deaf T :valuex 98 :w 10
      :text DET-MBOX3  :label "Date/Version:" :deaf T :valuex 98 :w 10
      :next-row T
      :text DET-MBOX4  :label "MailBox 4:" :deaf T :valuex 68 :w 10
      :text DET-MBOX5  :label "DMA buffer (P):" :deaf T :valuex 98 :w 10
      :text DET-MBOX6  :label "DMA buffer (S):" :deaf T :valuex 98 :w 10
      :text DET-MBOX7  :label "MailBox 7:" :deaf T :valuex 98 :w 10

   :panel DOORBELLS  :layout T :padx 10 :pady 10
      :layout H; :row-gap 0; :col-gap 10
      :message DET-DOORBELLS :label "Doorbells:" :bold T
      :text DET-P2S  :label "PCI to Sharc:" :valuex 150 :w 10
      :col-gap 20
      :text DET-S2P  :label "Sharc to PCI:" :valuex 84 :w 10

   :panel DATACOUNTERS  :layout T :padx 10 :pady 10
      :layout H; :row-gap 0; :col-gap 10
      :message DET-DOORBELLS :label "Collator Counters:" :bold T
      :text DET-PRODUCED  :label "Produced:" :deaf T :valuex 118 :w 10
      :text DET-CONSUMED  :label "Consumed:" :deaf T :valuex 98 :w 10

#   :panel STOPHERE  :layout T :padx 10 :pady 10
#      :layout H; :row-gap 0; :col-gap 10
#      :message DET-STOPHERE :label "Stop Data Flow:" :bold T
#      :col-gap 0
#      :message DET-DMABUFF :label "DMA Buffer"
#      :checkbox DET-DMA2LINKQ :label "" :value 0
#      :message DET-LINKQ :label "Link Queue"
#      :checkbox DET-LINKQ2OLDEST :label "" :value 0
#      :message DET-OLDEST :label "Oldest"
#      :checkbox DET-OLDEST2COLLATE :label "" :value 0
#      :message DET-COLLATE :label "Collate Buffer"
#      :checkbox DET-COLLATE2MERGE :label "" :value 0
#      :message DET-MERGE :label "Merge Buffer"
#      :message DET-PASS :label "Pass Data Unsorted"
#      :checkbox DET-PASSTHRU :label "" :value 0

#   :panel MEMORY  :layout T :padx 10 :pady 10
#      :layout H; :row-gap 0; :col-gap 10
#      :message DET-MES1  :label "Inspect SHARC memory" :bold T
#      :next-row T
#      :text DET-MEMADDR  :label " Address:" :value 0x000000 :w 10
#      :text DET-MEMCONT  :label "Contents:" :deaf T :w 10
#      :button DET-READ :label "Read" :command "update-sharcmem"
#      :col-gap 2
#      :message DET-MES2  :label "Change Address:"
#      :button DET-MEMDEC  :label "Dec" :command "do-memmod -1;update-sharcmem"
#      :button DET-MEMINC  :label "Inc" :command "do-memmod 1;update-sharcmem"

   :panel DISPLAY :layout B :padx 10 :after FOOTER
      :layout H; :row-gap 10; :col-gap 15
      :choice-stack DET-SOURCE :label "Read From:" :strings $buffnames :value $SourceIndex
      :choice DET-LINK :label "Link" :strings [list 0 1 2 3 4 5] :exclusive T
      :text DET-MEMOFFSET :label "Offset:" :w 8 :value "$memOffset"
# :x 200
      :button DET-ZERO :label "Zero Offset" :command "zero_offset"
      :checkbox DET-INCR :label "Increment Offset" :value 0
#      :text DET-OVERRUN :label "Link Overruns:" :w 6 :deaf T
      :next-row T
      :galley DET-MEMDISPLAY :rows 11 :cols 90 :font fixed

   :panel CLICK :layout B :padx 10 :pady 10 :after FOOTER
      :layout H; :row-gap 15; :col-gap 15
      :button DET-REDISPLAY :label "Redisplay"
      :button DET-RDBUFF :label $label :command "click-readBuff"
      :button DET-RDCURR :label "Read Current" :command "click-readCurr"
      :button DET-FORMAT :label "Format Data"  :command format_the_dump
      :button DET-EMPTY  :label "Empty memory display"
      :button DET-DEFINE :label "Define Registers" :command "click-define-asp-registers"

   :panel P21 :layout T :padx 5 :pady 5

  :show T
 set started 1
   if {$SourceIndex==3} then {
      :modify DET-RDBUFF :inactive T
   } else {
      :modify DET-RDBUFF :inactive F
   }
   if {$SourceIndex==3} then {
      :modify DET-RDCURR :inactive T
   } else {
      :modify DET-RDCURR :inactive F
   }
}

proc get-buffers-sent {args} {
   global sname CurrentProc EGallow
   
   if {[llength $args]} {set p [first $args]} {set p mem}

   set keepproc $CurrentProc
   set CurrentProc $p

   set z [catch {ersread $sname.MailBox1} m]
   if {$z != 0 && $m != "0x10002"} then {
      midas-error $m
   }
   set CurrentProc $keepproc
   return $m
}

# This procedure writes 1 to the Sharc doorbell - use with care
proc initialise-links {args} {
   global sname CurrentASP CurrentProc EGreport EGallow

   catch {do-define-asp-registers $CurrentASP}
   if {[llength $args]} {set p [first $args]} {set p mem}

   set keepproc $CurrentProc
   set CurrentProc $p

   set egallow $EGallow; set EGallow 0x10002
   set egreport $EGreport; set EGreport -1
   set z [catch {write-sharc PCI2Sharc 1} m]
   if {$z != 0} then {
      midas-error $m
   }
   set EGreport $egreport
   set EGallow $egallow
   set CurrentProc $keepproc
   return
}

proc update-sharc {} {
   global sname EGallow

   for {set i 0} {$i<8} {incr i} {
      set v [ersread $sname.MailBox$i]
      :modify DET-MBOX$i  :value $v
   }
   set v [ersread $sname.PCI2Sharc]
   :modify DET-P2S  :value $v
   set v [ersread $sname.Sharc2PCI]
   :modify DET-S2P  :value $v
}

proc update-datacounters {} {
   global EGallow cname CurrentASP
   global CurrentProc MergeProc mname

   set TempProc $CurrentProc
   set CurrentProc $MergeProc
   set v [ersread $mname.Asp$CurrentASP.Produced]
   :modify DET-PRODUCED  :value $v
   set v [ersread $mname.Asp$CurrentASP.Consumed]
   :modify DET-CONSUMED  :value $v
   set CurrentProc $TempProc
}

proc update-others {} {
   global memOffset meminc sname SourceIndex

   if {$SourceIndex == 5} then {
#      set o [ersread $sname.Link.OverRun$LinkIndex]
      set m 0x0000
      set i 0
      :modify DET-MEMOFFSET  :deaf T
   } else {
      set o "******"
      set m $memOffset
      set i $meminc
   }

#   :modify DET-OVERRUN  :value $o
   :modify DET-MEMOFFSET  :value $m
   :modify DET-INCR  :value $i
}

proc xupdate-stophere {} {
   global cname CurrentASP

   set v [ersread $cname.Asp$CurrentASP.StopHere]
   switch  $v {
      0 {
            :modify DET-DMA2LINKQ      :value 0
            :modify DET-LINKQ2OLDEST   :value 0
            :modify DET-OLDEST2COLLATE :value 0
            :modify DET-COLLATE2MERGE  :value 0
            :modify DET-PASSTHRU       :value 0
      }

      1 {
            :modify DET-DMA2LINKQ      :value 1
            :modify DET-LINKQ2OLDEST   :value 0
            :modify DET-OLDEST2COLLATE :value 0
            :modify DET-COLLATE2MERGE  :value 0
            :modify DET-PASSTHRU       :value 0
      }

      2 {
            :modify DET-DMA2LINKQ      :value 0
            :modify DET-LINKQ2OLDEST   :value 1
            :modify DET-OLDEST2COLLATE :value 0
            :modify DET-COLLATE2MERGE  :value 0
            :modify DET-PASSTHRU       :value 0
      }

      3 {
            :modify DET-DMA2LINKQ      :value 0
            :modify DET-LINKQ2OLDEST   :value 0
            :modify DET-OLDEST2COLLATE :value 1
            :modify DET-COLLATE2MERGE  :value 0
            :modify DET-PASSTHRU       :value 0
      }

      4 {
            :modify DET-DMA2LINKQ      :value 0
            :modify DET-LINKQ2OLDEST   :value 0
            :modify DET-OLDEST2COLLATE :value 0
            :modify DET-COLLATE2MERGE  :value 1
            :modify DET-PASSTHRU       :value 0
      }

      5 {
            :modify DET-DMA2LINKQ      :value 0
            :modify DET-LINKQ2OLDEST   :value 0
            :modify DET-OLDEST2COLLATE :value 0
            :modify DET-COLLATE2MERGE  :value 0
            :modify DET-PASSTHRU       :value 1
      }

      default {
            :modify DET-DMA2LINKQ      :value 0
            :modify DET-LINKQ2OLDEST   :value 0
            :modify DET-OLDEST2COLLATE :value 0
            :modify DET-COLLATE2MERGE  :value 0
            :modify DET-PASSTHRU       :value 0
            erswrite $cname.Asp$CurrentASP.StopHere 0
      }

   }
}

proc make-processor-list {} {
  global started
  global ProcessorList HostList resource
  global CurrentProc CurrentHost ProcIndex

#    find processors available

   if {[catch {eg "(experiment ?resources)"} m]} then {
      midas-warning "no experiment connected!!!"
      set ProcessorList {mem1}
      set HostList {mem1}
      set ProcIndex 0
   } else {
      set ProcessorList [make-sharc-resource-list "name" $m $resource]
      set HostList  [make-sharc-resource-list "host" $m $resource]
      if {$ProcIndex >= [llength $ProcessorList]} then {set ProcIndex 0}
   }
#    sort lists into sensible order
   set proclist [lsort $ProcessorList]
   set hostlist {}
   foreach c $proclist {
      lappend hostlist [lindex $HostList [lsearch $ProcessorList $c]]
   }
   set ProcessorList $proclist
   set HostList $hostlist
   set CurrentProc     [lindex $ProcessorList $ProcIndex]
   set CurrentHost     [lindex $HostList $ProcIndex]

   if {$started} then {
         :modify DET-PROCESSOR  :value $CurrentProc
#      :modify DET-PROCESSOR :strings $ProcessorList :value $ProcIndex
         :modify DET-HOST  :value $CurrentHost
   }
}

proc make-sharc-resource-list {class rlist resource} {
   set ll {}
   foreach item $rlist {
      set l [find-resource $class $item $resource]
      if {$l!={}} then {lappend ll $l}
   }
   return $ll
}

proc find-resource {class ps resource} {
   set l {}
   set role {}; set name {}; set host {}
   foreach p $ps {
      switch [first $p] {
         name {set name [second $p]}
         role {set role [second $p]}
         host {set host [second $p]}
         type {set type [second $p]}
         default {}
      }
   }
   if {$role==$resource && $type=="register" && $name!="merge"}  then  {set l [set $class]}
   return $l
}

proc ersdefine {register class} {
   global CurrentProc
   EG "register :name '/$CurrentProc/$register' :class '$class' !new"
}

proc ersinquire {register} {
   global CurrentProc
   eg "(register :pattern '/$CurrentProc/$register' ?names)"
}

proc erswta {register args} {
   global CurrentProc
   EG "register  :name '/$CurrentProc/$register' :object '$args'"
}

proc ersread {register} {
   global CurrentProc EGreport

   set egreport $EGreport; set EGreport -1
   return [string trimright [EG "register  :name '/$CurrentProc/$register' ?value"]]
   set EGreport $egreport
}

# Beware this uses :pattern - so it could write to ALL hosts with 1 call
proc erswrite {register val} {
   global debug CurrentProc
   if {$debug} then {puts stderr "erswrite: EG register  :pattern '/$CurrentProc/$register' :value '$val'"}
   EG "register  :pattern '/$CurrentProc/$register' :value '$val'"
}

proc define-register {name class attributes} {
   global sname  EGreport
   global CurrentProc

   set register $sname.$name

   set z [catch {ersinquire $register} r]
   if {$z != 0} then {return}
   if {$r==""} then {
      if {$EGreport>=0} then {set-footer "defining register $register"}
      if {[catch {ersdefine $register $class} m]} then {
         midas-error $m
      }
   }
   set z [catch {ersinquire $register} r]
   if {$r == $register} then {erswta $register "$attributes"}
   clear-footer
}

proc define-asp-registers {p a} {
   global sname CurrentProc barRegisters ASPC2Registers
   set keepproc $CurrentProc
   set CurrentProc $p

   foreach r $barRegisters {
      set attr [list [third $r] asp[set a][first $r] [fourth $r] [fifth $r] [sixth $r]]
      define-register asp[set a][first $r] [second $r] $attr
   }
   set pctl [ersread $sname.asp[set a]_PCTL]
   set sctl [ersread $sname.asp[set a]_SCTL]
   set dma  [ersread $sname.asp[set a]_DMA]

   foreach r $ASPC2Registers {
      set attr [lreplace $r 0 2]
      set addr [expr $[third $r] + [second $attr]]
      set attr [lreplace $attr 1 1 $addr]
      define-register [first $r] [second $r] $attr
   }

   set CurrentProc $keepproc
}

proc do-define-asp-registers {a} {
   global ProcessorList CurrentProc sname ASPC2Registers EGreport
   set egreport $EGreport; set EGreport -1

   define-asp-registers $CurrentProc $a

   set EGreport $egreport
}
