# 
# remote control interface for the Agilent Infiniium Oscilloscope

loadsource Scope2

set frame "Scope"
set frametitle "Agilent Infiniium Oscilloscope"
set icontitle "Scope"

if {[info exists env(MIDASScopeSpectrumServer)] == 0} then {set SpectrumServer MemSas} \
    else {set SpectrumServer $env(MIDASScopeSpectrumServer)}
    
if {[info exists env(MIDASScopeRegisterServer)] == 0} then {set RegisterServer Scope} \
    else {set RegisterServer $env(MIDASScopeRegisterServer)}

##


set mname "scope"

set nchannels 4

set tbase 0
set delay 0
set tlev 0
set tchan 1
set tpos 0
set sweep 0
for {set i 1} {$i <= $nchannels} {incr i} {
    set chan($i) 0
    set scale($i) 0
    set offset($i) 0
    set ohm($i) 0
}


proc make-frame {} {
  global env nchannels
  global tbase delay tlev tchan tpos sweep chan scale offset ohm

  :frame [self frame-name] :label "[self frame-title]" :show-footer T \
      :icon-name "[self icon-title]"

   :panel MODULE :layout T :padx 10 :pady 10
     :layout H; :row-gap 10; :col-gap 20

      :next-row T
      :hskip 25
      :message M0 :label "Timebase"  :bold T
      :text TIMEBASE :label "TimeBase (ns)" :w 10 :value $tbase
      :text DELAY :label "Delay (ns)" :w 10 :value 20 :value $delay

      :hskip 330
      :button SETUP    :label "Connect to Scope"     :command Setup_Scope

      :next-row T
      :hskip 25
      :message M9 :label "Trigger"  :bold T
        :slider TLEV :label "Level (mV)" :valuex 2c :w 3 :min -5000 :max 5000 :value $tlev
	:choice TCHAN :label "Channel" :strings "1 2 3 4" :value [expr ($tchan-1)] :w 5
	:choice TPOS :label "Slope" :strings "Pos Neg" :value $tpos :w 5
	:choice SWEEP :label "Mode" :strings "Auto Triggered Single" :value $sweep :w 5

      :next-row T
      :hskip 25
      :message M1 :label "Channels"  :bold T
      :hskip 25
      for {set i 1} {$i <= $nchannels} {incr i} {
        :checkbox CHAN$i :label "$i" :value $chan($i)
	}

      :next-row T
      :hskip 25
      :message M2 :label "Channel Scale (mV/div)                    Channel Offset (mV)"  :bold T
      :next-row T
      for {set i 1} {$i <= $nchannels} {incr i} {
        :slider SCALE$i :label Channel$i: :valuex 2c :w 3 :min 0 :max 5000 :value $scale($i)
        :slider OFFSET$i :valuex 2c :w 3 :min -5000 :max 5000 :value $offset($i)
	:choice OHM$i :strings "1Mohm 50ohm" :w 5 :value $ohm($i)
      :next-row T
      }

   :panel CLICK :layout T :padx 10
      :layout H; :col-gap 30

      :button REDISPLAY :label Redisplay 
      :hskip 40

   :show T
}

proc get-setup {} {
 global nchannels
 global tbase delay tchan tlev tpos sweep chan scale offset ohm

 set X [Query_Scope ":TIMEbase:SCALe?"]
 set tbase [format "%.0f" [expr ($X*1000000000)]] 

 set X [Query_Scope ":TIMEbase:DELay?"]
 set delay [format "%.0f" [expr ($X*1000000000)]] 

 set X [Query_Scope ":TRIGger:EDGE:SOURce?"]
 set tchan [string range $X 4 end]

 set X [Query_Scope ":TRIGger:LEVel? CHANnel$tchan"]
 set tlev [format "%.0f" [expr ($X*1000)]] 

 set X [Query_Scope ":TRIGger:EDGE:SLOPe?"]
 if {$X == "POS"} then {set tpos 0} else {set tpos 1}
 
 set X [Query_Scope ":TRIGger:SWEep?"]
 if {$X == "AUTO"} then {set sweep 0}
 if {$X == "TRIG"} then {set sweep 1}
 if {$X == "SING"} then {set sweep 2}

 for {set channel 1} {$channel <= $nchannels} {incr channel} {

 set X [Query_Scope ":CHANNEL$channel:DISPlay?"]
 set chan($channel) $X

 set X [Query_Scope ":CHANnel$channel:SCALe?"]
 set scale($channel) [format "%.0f" [expr ($X*1000)]] 

 set X [Query_Scope ":CHANnel$channel:OFFSet?"]
 set offset($channel) [format "%.0f" [expr ($X*1000)]] 

 set X [Query_Scope ":CHANNEL$channel:INPut?"]
 if {$X == "DC"} then {set ohm($channel) 0} else {set ohm($channel) 1}

 }

}

proc click-timebase {v} {
   global tbase
   set tbase $v
   set V [format "%dN" $v] 
   do-scope ":TIMebase:SCALe $V"
}

proc click-delay {v} {
   global delay
   set delay $v
   set V [format "%dN" $v] 
   do-scope ":TIMebase:DELay $V"
}

proc click-tlev {v} {
   global tchan tlev
   set tlev $v
   set V [format "%dM" $v] 
   do-scope ":TRIGger:LEVel CHANnel$tchan,$V"
}

proc click-tchan {v} {
   global tchan tlev tpos
   set tchan [expr ($v+1)]
   do-scope ":TRIGger:EDGE:SOURce CHANnel$tchan"

   set X [Query_Scope ":TRIGger:LEVel? CHANnel$tchan"]
   set tlev [format "%.0f" [expr ($X*1000)]] 
   :modify TLEV :value $tlev

   set X [Query_Scope ":TRIGger:EDGE:SLOPe?"]
   if {$X == "POS"} then {set tpos 0} else {set tpos 1}
   :modify TPOS :value $tpos
}

proc click-tpos {v} {
   global tpos
   set tpos $v
   if {$v == 0} then {set V "POSitive"} else {set V "NEGative"}
   do-scope ":TRIGger:EDGE:SLOPe $V"
}

proc click-sweep {v} {
   global sweep
   set sweep $v
   if {$v == 0} then {set M "AUTO"}
   if {$v == 1} then {set M "TRIGgered"}
   if {$v == 2} then {set M "SINGle"}
   do-scope ":TRIGger:SWEep $M"
}


proc click-chan {w v} {
   global chan
   set channel [string range $w 4 end]
   set chan($channel) $v
   do-scope ":CHANNEL$channel:DISPlay $v"
}


proc click-scale {w v} {
   global scale
   set channel [string range $w 5 end]
   set scale($channel) $v
   set V [format "%dM" $v] 
   do-scope ":CHANnel$channel:SCALe $V"
}

proc click-offset {w v} {
   global offset
   set channel [string range $w 6 end]
   set offset($channel) $v
   set V [format "%dM" $v] 
   do-scope ":CHANnel$channel:OFFSet $V"
}

proc click-ohm {w v} {
   global ohm
   set channel [string range $w 3 end]
   set ohm($channel) $v
   if {$v == 0} then {set V "DC"} else {set V "DC50"}
   do-scope ":CHANNEL$channel:INPut $V"
}

proc do-scope {message} {
#   puts stderr "--> $message"
    Command_Scope "$message"
}

proc click arg {
   global debug
   if {$debug} then {puts stderr "click with argument \{$arg\}"}
   set f [lindex $arg 0]
   set c [lindex $arg 1]
   set w [lindex $arg 2]
   set v [lindex $arg 3]
   if {$c==":quit"} then {click-quit; return}
   switch -glob $w {

      TIMEBASE {click-timebase $v}
      DELAY    {click-delay $v}

      TLEV  {click-tlev $v}
      TCHAN  {click-tchan $v}
      TPOS  {click-tpos $v}
      SWEEP  {click-sweep $v}


      SCALE*  {click-scale $w $v}
      OFFSET*  {click-offset $w $v}
      CHAN*  {click-chan $w $v}
      OHM*  {click-ohm $w $v}

      REDISPLAY {click-redisplay}
      default   {puts stdout $arg}
   }
}



proc click-redisplay {} {
  global nchannels
  global tbase delay tlev tchan tpos sweep chan scale offset ohm

  get-setup
  :modify TIMEBASE :value $tbase
  :modify DELAY :value $delay
  :modify TLEV :value $tlev
  :modify TCHAN :value [expr ($tchan-1)]
  :modify TPOS :value $tpos
  :modify SWEEP :value $sweep

 for {set channel 1} {$channel <= $nchannels} {incr channel} {
     :modify SCALE$channel :value $scale($channel)
     :modify OFFSET$channel :value $offset($channel)
     :modify CHAN$channel :value $chan($channel)
     :modify OHM$channel :value $ohm($channel)
 }



}





#
#
#



#proc insert {s} {puts stderr "xx $s\n"}
proc insert {s} {}

proc start args {
   global started
   if {$started == 0} then {
      make-frame
      click-redisplay
      set started 1
   } else {open-frame}
}

proc Setup_Scope args {
    Create_Scope
    Open_Scope
}

